package cn.ibizlab.service.dictapi.resource;

import java.sql.Timestamp;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.math.BigInteger;
import java.util.HashMap;

import lombok.extern.slf4j.Slf4j;
import com.alibaba.fastjson.JSONObject;

import javax.servlet.ServletRequest;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.HttpStatus;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.StringUtils;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

import cn.ibizlab.core.dict.service.dto.IBZDictItemDTO;
import cn.ibizlab.core.dict.domain.IBZDictItem;
import cn.ibizlab.core.dict.service.IIBZDictItemService;
import cn.ibizlab.core.dict.filter.IBZDictItemSearchContext;
import cn.ibizlab.core.dict.service.mapping.IBZDictItemMapping;


import cn.ibizlab.util.log.IBIZLog;

@Slf4j
@IBIZLog
@Api(tags = {"IBZDictItem" })
@RestController
@RequestMapping("")
public class IBZDictItemResource {

    @Autowired
    private IIBZDictItemService ibzdictitemService;

    @ApiOperation(value = "Get", tags = {"IBZDictItem" },  notes = "Get")
	@RequestMapping(method = RequestMethod.GET, value = "/dictapi/ibzdictitems/{ibzdictitem_id}")
    public ResponseEntity<IBZDictItemDTO> get(@PathVariable("ibzdictitem_id") String ibzdictitem_id) {
        IBZDictItem domain = ibzdictitemService.get(ibzdictitem_id);
        IBZDictItemDTO dto = IBZDictItemMapping.MAPPER.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "GetDraft", tags = {"IBZDictItem" },  notes = "GetDraft")
	@RequestMapping(method = RequestMethod.GET, value = "/dictapi/ibzdictitems/getdraft")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> getDraft() {
        return ResponseEntity.status(HttpStatus.OK).body(new IBZDictItemDTO());
    }
                //SAVE
    @ApiOperation(value = "Save", tags = {"IBZDictItem" },  notes = "Save")
	@RequestMapping(method = RequestMethod.POST, value = "/dictapi/ibzdictitems/{ibzdictitem_id}/save")
    public ResponseEntity<Boolean> save(@RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem ibzdictitem = IBZDictItemMapping.MAPPER.toDomain(ibzdictitemdto);
    	Boolean b = ibzdictitemService.save(ibzdictitem) ;
        return ResponseEntity.status(HttpStatus.OK).body(b);
    }

    @ApiOperation(value = "CheckKey", tags = {"IBZDictItem" },  notes = "CheckKey")
	@RequestMapping(method = RequestMethod.POST, value = "/dictapi/ibzdictitems/checkkey")
    public ResponseEntity<Boolean> checkKey(@RequestBody IBZDictItemDTO ibzdictitemdto) {
        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "Remove", tags = {"IBZDictItem" },  notes = "Remove")
	@RequestMapping(method = RequestMethod.DELETE, value = "/dictapi/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<Boolean> remove(@PathVariable("ibzdictitem_id") String ibzdictitem_id) {
        IBZDictItemDTO ibzdictitemdto = new IBZDictItemDTO();
		IBZDictItem domain = new IBZDictItem();
		ibzdictitemdto.setItemId(ibzdictitem_id);
		domain.setItemId(ibzdictitem_id);
        Boolean rst = ibzdictitemService.remove(domain.getItemId());
        if(rst){
			return ResponseEntity.status(HttpStatus.OK).body(rst);
		}else{
			return ResponseEntity.status(HttpStatus.OK).body(rst);
		}
    }

    @ApiOperation(value = "Create", tags = {"IBZDictItem" },  notes = "Create")
	@RequestMapping(method = RequestMethod.POST, value = "/dictapi/ibzdictitems")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> create(@RequestBody IBZDictItemDTO ibzdictitemdto) {
        IBZDictItem domain = IBZDictItemMapping.MAPPER.toDomain(ibzdictitemdto);
		ibzdictitemService.create(domain);
        IBZDictItemDTO dto = IBZDictItemMapping.MAPPER.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "Update", tags = {"IBZDictItem" },  notes = "Update")
	@RequestMapping(method = RequestMethod.PUT, value = "/dictapi/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> update(@PathVariable("ibzdictitem_id") String ibzdictitem_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
		IBZDictItem domain = IBZDictItemMapping.MAPPER.toDomain(ibzdictitemdto);
        domain.setItemId(ibzdictitem_id);
		ibzdictitemService.update(domain);
		IBZDictItemDTO dto = IBZDictItemMapping.MAPPER.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

            //9
	@ApiOperation(value = "获取DEFAULT", tags = {"IBZDictItem" } ,notes = "获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/dictapi/ibzdictitems/fetchdefault")
	public ResponseEntity<List<IBZDictItemDTO>> fetchDefault(IBZDictItemSearchContext context) {
        List<IBZDictItemDTO> list = new ArrayList<IBZDictItemDTO>();
        
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
        for(IBZDictItem ibzdictitem : domains.getContent()){
            IBZDictItemDTO dto =IBZDictItemMapping.MAPPER.toDto(ibzdictitem);
            list.add(dto);
        }
        return ResponseEntity.status(HttpStatus.OK)
                .header("x-page", String.valueOf(context.getPageable().getPageNumber()))
                .header("x-per-page", String.valueOf(context.getPageable().getPageSize()))
                .header("x-total", String.valueOf(domains.getTotalElements()))
                .body(list);
	}



    @ApiOperation(value = "Get", tags = {"IBZDictItem" },  notes = "Get")
	@RequestMapping(method = RequestMethod.GET, value = "/dictapi/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitem_id}")
    public ResponseEntity<IBZDictItemDTO> getByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id) {
        IBZDictItem domain = ibzdictitemService.get(ibzdictitem_id);
        IBZDictItemDTO dto = IBZDictItemMapping.MAPPER.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "GetDraft", tags = {"IBZDictItem" },  notes = "GetDraft")
	@RequestMapping(method = RequestMethod.GET, value = "/dictapi/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitemitemid}/getdraft")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> getDraftByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
                    //8
        IBZDictItem ibzdictitem = IBZDictItemMapping.MAPPER.toDomain(ibzdictitemdto);
    	ibzdictitem = ibzdictitemService.getDraft(ibzdictitem) ;
        ibzdictitemdto = IBZDictItemMapping.MAPPER.toDto(ibzdictitem);
        return ResponseEntity.status(HttpStatus.OK).body(ibzdictitemdto);
    }

    @ApiOperation(value = "Save", tags = {"IBZDictItem" },  notes = "Save")
	@RequestMapping(method = RequestMethod.POST, value = "/dictapi/ibzdicts/{ibzdict_id}/ibzdictitems/save")
    public ResponseEntity<Boolean> saveByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
                //6
        IBZDictItem domain = IBZDictItemMapping.MAPPER.toDomain(ibzdictitemdto);
        domain.setDictId(ibzdict_id);
    	Boolean b = ibzdictitemService.save(domain) ;
        return ResponseEntity.status(HttpStatus.OK).body(b);
    }

    @ApiOperation(value = "CheckKey", tags = {"IBZDictItem" },  notes = "CheckKey")
	@RequestMapping(method = RequestMethod.POST, value = "/dictapi/ibzdicts/{ibzdict_id}/ibzdictitems/checkkey")
    public ResponseEntity<Boolean> checkKeyByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {

        return  ResponseEntity.status(HttpStatus.OK).body(true);
    }

    @ApiOperation(value = "Remove", tags = {"IBZDictItem" },  notes = "Remove")
	@RequestMapping(method = RequestMethod.DELETE, value = "/dictapi/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<Boolean> removeByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id) {
		IBZDictItem domain = new IBZDictItem();
		domain.setItemId(ibzdictitem_id);
        Boolean rst = ibzdictitemService.remove(domain.getItemId());
        if(rst){
			return ResponseEntity.status(HttpStatus.OK).body(rst);
		}else{
			return ResponseEntity.status(HttpStatus.OK).body(rst);
		}
    }

    @ApiOperation(value = "Create", tags = {"IBZDictItem" },  notes = "Create")
	@RequestMapping(method = RequestMethod.POST, value = "/dictapi/ibzdicts/{ibzdict_id}/ibzdictitems")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> createByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
                //4
        IBZDictItem domain = IBZDictItemMapping.MAPPER.toDomain(ibzdictitemdto);
        domain.setDictId(ibzdict_id);
		ibzdictitemService.create(domain);
        IBZDictItemDTO dto = IBZDictItemMapping.MAPPER.toDto(domain);
		return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

    @ApiOperation(value = "Update", tags = {"IBZDictItem" },  notes = "Update")
	@RequestMapping(method = RequestMethod.PUT, value = "/dictapi/ibzdicts/{ibzdict_id}/ibzdictitems/{ibzdictitem_id}")
    @Transactional
    public ResponseEntity<IBZDictItemDTO> updateByIBZDict(@PathVariable("ibzdict_id") String ibzdict_id, @PathVariable("ibzdictitem_id") String ibzdictitem_id, @RequestBody IBZDictItemDTO ibzdictitemdto) {
                //5
        IBZDictItem domain = IBZDictItemMapping.MAPPER.toDomain(ibzdictitemdto);
        domain.setDictId(ibzdict_id);
        domain.setItemId(ibzdictitem_id);
		ibzdictitemService.update(domain);
        IBZDictItemDTO dto = IBZDictItemMapping.MAPPER.toDto(domain);
        return ResponseEntity.status(HttpStatus.OK).body(dto);
    }

            //10
	@ApiOperation(value = "获取DEFAULT", tags = {"IBZDictItem" } ,notes = "获取DEFAULT")
    @RequestMapping(method= RequestMethod.GET , value="/dictapi/ibzdicts/{ibzdict_id}/ibzdictitems/fetchdefault")
	public ResponseEntity<Page<IBZDictItemDTO>> fetchIBZDictItemDefault(@PathVariable("ibzdict_id") String ibzdict_id,IBZDictItemSearchContext context,Pageable pageable ,ServletRequest request) {
        context.setPageable(pageable);
        List<IBZDictItemDTO> list = new ArrayList<IBZDictItemDTO>();
        context.setN_dictid_eq(ibzdict_id);
        Page<IBZDictItem> domains = ibzdictitemService.searchDefault(context) ;
        for(IBZDictItem ibzdictitem : domains.getContent()){
            IBZDictItemDTO dto = IBZDictItemMapping.MAPPER.toDto(ibzdictitem);
            list.add(dto);
        }
	    return ResponseEntity.status(HttpStatus.OK).body(new PageImpl(list,context.getPageable(),domains.getTotalElements()));
	}


}
